@@ -105,16 +105,11 @@ Options:\n\
                             (default: %s)\n\
   --ovnsb-db=DATABASE       connect to ovn-sb database at DATABASE\n\
                             (default: %s)\n\
-  --ic-nb-db=DATABASE       connect to ovn-ic-nb database at DATABASE\n\
-                            (default: %s)\n\
-  --ic-sb-db=DATABASE       connect to ovn-ic-sb database at DATABASE\n\
-                            (default: %s)\n\
   --unixctl=SOCKET          override default control socket name\n\
   -h, --help                display this help message\n\
   -o, --options             list available options\n\
   -V, --version             display version information\n\
-", program_name, program_name, default_nb_db(), default_sb_db(),
-    default_ic_nb_db(), default_ic_sb_db());
+", program_name, program_name, default_nb_db(), default_sb_db());
     daemon_usage();
     vlog_usage();
     stream_usage("database", true, true, false);
@@ -131,18 +126,14 @@ az_run(struct ic_context *ctx)
         return NULL;
     }
 
-    /* Update old AZ if name changes.  Note: if name changed when ovn-ic
-     * is not running, one has to manually delete/update the old AZ with:
+    /* Delete old AZ if name changes.  Note: if name changed when ovn-ic
+     * is not running, one has to manually delete the old AZ with:
      * "ovn-ic-sbctl destroy avail <az>". */
     static char *az_name;
     const struct icsbrec_availability_zone *az;
     if (az_name && strcmp(az_name, nb_global->name)) {
         ICSBREC_AVAILABILITY_ZONE_FOR_EACH (az, ctx->ovnisb_idl) {
-            /* AZ name update locally need to update az in ISB. */
-            if (nb_global->name[0] && !strcmp(az->name, az_name)) {
-                icsbrec_availability_zone_set_name(az, nb_global->name);
-                break;
-            } else if (!nb_global->name[0] && !strcmp(az->name, az_name)) {
+            if (!strcmp(az->name, az_name)) {
                 icsbrec_availability_zone_delete(az);
                 break;
             }
@@ -323,10 +314,6 @@ sync_isb_gw_to_sb(struct ic_context *ctx,
 {
     sbrec_chassis_set_hostname(chassis, gw->hostname);
     sbrec_chassis_update_other_config_setkey(chassis, "is-remote", "true");
-    /* TODO(lucasagomes): Continue writing the configuration to the
-     * external_ids column for backward compatibility with the current
-     * systems, this behavior should be removed in the future. */
-    sbrec_chassis_update_external_ids_setkey(chassis, "is-remote", "true");
 
     /* Sync encaps used by this gateway. */
     ovs_assert(gw->n_encaps);
@@ -1062,15 +1049,12 @@ prefix_is_black_listed(const struct smap *nb_options,
                 continue;
             }
         } else {
-            struct in6_addr mask = ipv6_create_mask(plen);
-            /* First calculate the difference between bl_prefix and prefix, so
-             * use the bl mask to ensure prefixes are correctly validated.
-             * e.g.: 2005:1734:5678::/50 is a subnet of 2005:1234::/21 */
-            struct in6_addr m_prefixes = ipv6_addr_bitand(prefix, &bl_prefix);
-            struct in6_addr m_prefix = ipv6_addr_bitand(&m_prefixes, &mask);
-            struct in6_addr m_bl_prefix = ipv6_addr_bitand(&bl_prefix, &mask);
-            if (!ipv6_addr_equals(&m_prefix, &m_bl_prefix)) {
-                continue;
+            struct in6_addr mask = ipv6_create_mask(bl_plen);
+            for (int i = 0; i < 16 && mask.s6_addr[i] != 0; i++) {
+                if ((prefix->s6_addr[i] & mask.s6_addr[i])
+                    != (bl_prefix.s6_addr[i] & mask.s6_addr[i])) {
+                    continue;
+                }
             }
         }
         matched = true;
@@ -1434,8 +1418,8 @@ sync_learned_routes(struct ic_context *ctx,
     }
 
     /* Delete extra learned routes. */
-    struct ic_route_info *route_learned, *next;
-    HMAP_FOR_EACH_SAFE (route_learned, next, node, &ic_lr->routes_learned) {
+    struct ic_route_info *route_learned;
+    HMAP_FOR_EACH_SAFE (route_learned, node, &ic_lr->routes_learned) {
         VLOG_DBG("Delete route %s -> %s that is not in IC-SB from NB.",
                  route_learned->nb_route->ip_prefix,
                  route_learned->nb_route->nexthop);
@@ -1509,8 +1493,8 @@ advertise_routes(struct ic_context *ctx,
     icsbrec_route_index_destroy_row(isb_route_key);
 
     /* Create the missing routes in IC-SB */
-    struct ic_route_info *route_adv, *next;
-    HMAP_FOR_EACH_SAFE (route_adv, next, node, routes_ad) {
+    struct ic_route_info *route_adv;
+    HMAP_FOR_EACH_SAFE (route_adv, node, routes_ad) {
         isb_route = icsbrec_route_insert(ctx->ovnisb_txn);
         icsbrec_route_set_transit_switch(isb_route, ts_name);
         icsbrec_route_set_availability_zone(isb_route, az);
@@ -1735,8 +1719,8 @@ route_run(struct ic_context *ctx,
     }
     icsbrec_port_binding_index_destroy_row(isb_pb_key);
 
-    struct ic_router_info *ic_lr, *next;
-    HMAP_FOR_EACH_SAFE (ic_lr, next, node, &ic_lrs) {
+    struct ic_router_info *ic_lr;
+    HMAP_FOR_EACH_SAFE (ic_lr, node, &ic_lrs) {
         advertise_lr_routes(ctx, az, ic_lr);
         sync_learned_routes(ctx, az, ic_lr);
         free(ic_lr->isb_pbs);
@@ -1927,113 +1911,13 @@ main(int argc, char *argv[])
     struct ovsdb_idl_loop ovnisb_idl_loop = OVSDB_IDL_LOOP_INITIALIZER(
         ovsdb_idl_create(ovn_ic_sb_db, &icsbrec_idl_class, true, true));
 
-    /* ovn-nb db. */
+    /* ovn-nb db. XXX: add only needed tables and columns */
     struct ovsdb_idl_loop ovnnb_idl_loop = OVSDB_IDL_LOOP_INITIALIZER(
-        ovsdb_idl_create(ovnnb_db, &nbrec_idl_class, false, true));
+        ovsdb_idl_create(ovnnb_db, &nbrec_idl_class, true, true));
 
-    ovsdb_idl_add_table(ovnnb_idl_loop.idl, &nbrec_table_nb_global);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl, &nbrec_nb_global_col_name);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl, &nbrec_nb_global_col_options);
-
-    ovsdb_idl_add_table(ovnnb_idl_loop.idl,
-                        &nbrec_table_logical_router_static_route);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_static_route_col_route_table);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_static_route_col_ip_prefix);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_static_route_col_nexthop);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_static_route_col_external_ids);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_static_route_col_options);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_static_route_col_policy);
-
-    ovsdb_idl_add_table(ovnnb_idl_loop.idl, &nbrec_table_logical_router);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_col_name);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_col_static_routes);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_col_ports);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_col_options);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_col_external_ids);
-
-    ovsdb_idl_add_table(ovnnb_idl_loop.idl, &nbrec_table_logical_router_port);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_port_col_name);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_port_col_networks);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_port_col_external_ids);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_port_col_options);
-
-    ovsdb_idl_add_table(ovnnb_idl_loop.idl, &nbrec_table_logical_switch);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_col_name);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_col_ports);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_col_other_config);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_col_external_ids);
-
-    ovsdb_idl_add_table(ovnnb_idl_loop.idl, &nbrec_table_logical_switch_port);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_name);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_addresses);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_options);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_type);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_up);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_addresses);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_enabled);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_switch_port_col_external_ids);
-
-    /* ovn-sb db. */
+    /* ovn-sb db. XXX: add only needed tables and columns */
     struct ovsdb_idl_loop ovnsb_idl_loop = OVSDB_IDL_LOOP_INITIALIZER(
-        ovsdb_idl_create(ovnsb_db, &sbrec_idl_class, false, true));
-
-    ovsdb_idl_add_table(ovnsb_idl_loop.idl, &sbrec_table_chassis);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_chassis_col_encaps);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_chassis_col_external_ids);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_chassis_col_name);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_chassis_col_hostname);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_chassis_col_other_config);
-
-    ovsdb_idl_add_table(ovnsb_idl_loop.idl, &sbrec_table_encap);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_encap_col_chassis_name);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_encap_col_type);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_encap_col_ip);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl, &sbrec_encap_col_options);
-
-    ovsdb_idl_add_table(ovnsb_idl_loop.idl, &sbrec_table_datapath_binding);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_datapath_binding_col_external_ids);
-
-    ovsdb_idl_add_table(ovnsb_idl_loop.idl, &sbrec_table_port_binding);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_port_binding_col_datapath);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_port_binding_col_mac);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_port_binding_col_options);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_port_binding_col_logical_port);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_port_binding_col_external_ids);
-    ovsdb_idl_add_column(ovnsb_idl_loop.idl,
-                         &sbrec_port_binding_col_chassis);
+        ovsdb_idl_create(ovnsb_db, &sbrec_idl_class, true, true));
 
     /* Create IDL indexes */
     struct ovsdb_idl_index *nbrec_ls_by_name
