@@ -41,20 +41,26 @@
 #include "unixctl.h"
 #include "util.h"
 #include "openvswitch/vlog.h"
+#include "lib/ovn-parallel-hmap.h"
 
 VLOG_DEFINE_THIS_MODULE(ovn_northd);
 
+static unixctl_cb_func ovn_northd_exit;
 static unixctl_cb_func ovn_northd_pause;
 static unixctl_cb_func ovn_northd_resume;
 static unixctl_cb_func ovn_northd_is_paused;
 static unixctl_cb_func ovn_northd_status;
 static unixctl_cb_func cluster_state_reset_cmd;
+static unixctl_cb_func ovn_northd_set_thread_count_cmd;
+static unixctl_cb_func ovn_northd_get_thread_count_cmd;
 
 struct northd_state {
     bool had_lock;
     bool paused;
 };
 
+#define OVN_MAX_SUPPORTED_THREADS 256
+
 static const char *ovnnb_db;
 static const char *ovnsb_db;
 static const char *unixctl_path;
@@ -99,12 +105,17 @@ static const char *rbac_fdb_update[] =
 static const char *rbac_port_binding_auth[] =
     {""};
 static const char *rbac_port_binding_update[] =
-    {"chassis", "encap", "up", "virtual_parent"};
+    {"chassis", "additional_chassis",
+     "encap", "additional_encap",
+     "up", "virtual_parent",
+     /* NOTE: we only need to update the additional-chassis-activated key,
+      * but RBAC_Role doesn't support mutate operation for subkeys. */
+     "options"};
 
 static const char *rbac_mac_binding_auth[] =
     {""};
 static const char *rbac_mac_binding_update[] =
-    {"logical_port", "ip", "mac", "datapath"};
+    {"logical_port", "ip", "mac", "datapath", "timestamp"};
 
 static const char *rbac_svc_monitor_auth[] =
     {""};
@@ -346,20 +357,20 @@ check_and_update_rbac(struct ovsdb_idl_txn *ovnsb_txn,
                       struct ovsdb_idl *ovnsb_idl)
 {
     const struct sbrec_rbac_role *rbac_role = NULL;
-    const struct sbrec_rbac_permission *perm_row, *perm_next;
-    const struct sbrec_rbac_role *role_row, *role_row_next;
+    const struct sbrec_rbac_permission *perm_row;
+    const struct sbrec_rbac_role *role_row;
     struct rbac_perm_cfg *pcfg;
 
     for (pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
         pcfg->row = NULL;
     }
 
-    SBREC_RBAC_PERMISSION_FOR_EACH_SAFE (perm_row, perm_next, ovnsb_idl) {
+    SBREC_RBAC_PERMISSION_FOR_EACH_SAFE (perm_row, ovnsb_idl) {
         if (!ovn_rbac_validate_perm(perm_row)) {
             sbrec_rbac_permission_delete(perm_row);
         }
     }
-    SBREC_RBAC_ROLE_FOR_EACH_SAFE (role_row, role_row_next, ovnsb_idl) {
+    SBREC_RBAC_ROLE_FOR_EACH_SAFE (role_row, ovnsb_idl) {
         if (strcmp(role_row->name, "ovn-controller")) {
             sbrec_rbac_role_delete(role_row);
         } else {
@@ -390,8 +401,8 @@ check_and_add_supported_dhcp_opts_to_sb_db(struct ovsdb_idl_txn *ovnsb_txn,
                     dhcp_opt_hash(supported_dhcp_opts[i].name));
     }
 
-    const struct sbrec_dhcp_options *opt_row, *opt_row_next;
-    SBREC_DHCP_OPTIONS_FOR_EACH_SAFE (opt_row, opt_row_next, ovnsb_idl) {
+    const struct sbrec_dhcp_options *opt_row;
+    SBREC_DHCP_OPTIONS_FOR_EACH_SAFE (opt_row, ovnsb_idl) {
         struct gen_opts_map *dhcp_opt =
             dhcp_opts_find(&dhcp_opts_to_add, opt_row->name);
         if (dhcp_opt) {
@@ -429,8 +440,8 @@ check_and_add_supported_dhcpv6_opts_to_sb_db(struct ovsdb_idl_txn *ovnsb_txn,
                     dhcp_opt_hash(supported_dhcpv6_opts[i].name));
     }
 
-    const struct sbrec_dhcpv6_options *opt_row, *opt_row_next;
-    SBREC_DHCPV6_OPTIONS_FOR_EACH_SAFE(opt_row, opt_row_next, ovnsb_idl) {
+    const struct sbrec_dhcpv6_options *opt_row;
+    SBREC_DHCPV6_OPTIONS_FOR_EACH_SAFE(opt_row, ovnsb_idl) {
         struct gen_opts_map *dhcp_opt =
             dhcp_opts_find(&dhcpv6_opts_to_add, opt_row->name);
         if (dhcp_opt) {
@@ -537,7 +548,7 @@ Options:\n\
   --ovnsb-db=DATABASE       connect to ovn-sb database at DATABASE\n\
                             (default: %s)\n\
   --dry-run                 start in paused state (do not commit db changes)\n\
-  --dummy-numa              override default NUMA node and CPU core discovery\n\
+  --n-threads=N             specify number of threads\n\
   --unixctl=SOCKET          override default control socket name\n\
   -h, --help                display this help message\n\
   -o, --options             list available options\n\
@@ -550,14 +561,14 @@ Options:\n\
 
 static void
 parse_options(int argc OVS_UNUSED, char *argv[] OVS_UNUSED,
-              bool *paused)
+              bool *paused, int *n_threads)
 {
     enum {
         OVN_DAEMON_OPTION_ENUMS,
         VLOG_OPTION_ENUMS,
         SSL_OPTION_ENUMS,
         OPT_DRY_RUN,
-        OPT_DUMMY_NUMA,
+        OPT_N_THREADS,
     };
     static const struct option long_options[] = {
         {"ovnsb-db", required_argument, NULL, 'd'},
@@ -567,7 +578,7 @@ parse_options(int argc OVS_UNUSED, char *argv[] OVS_UNUSED,
         {"options", no_argument, NULL, 'o'},
         {"version", no_argument, NULL, 'V'},
         {"dry-run", no_argument, NULL, OPT_DRY_RUN},
-        {"dummy-numa", required_argument, NULL, OPT_DUMMY_NUMA},
+        {"n-threads", required_argument, NULL, OPT_N_THREADS},
         OVN_DAEMON_LONG_OPTIONS,
         VLOG_LONG_OPTIONS,
         STREAM_SSL_LONG_OPTIONS,
@@ -623,8 +634,21 @@ parse_options(int argc OVS_UNUSED, char *argv[] OVS_UNUSED,
             ovn_print_version(0, 0);
             exit(EXIT_SUCCESS);
 
-        case OPT_DUMMY_NUMA:
-            ovs_numa_set_dummy(optarg);
+        case OPT_N_THREADS:
+            *n_threads = strtoul(optarg, NULL, 10);
+            if (*n_threads < 1) {
+                *n_threads = 1;
+                VLOG_WARN("Setting n_threads to %d as --n-threads option was "
+                    "set to : [%s]", *n_threads, optarg);
+            }
+            if (*n_threads > OVN_MAX_SUPPORTED_THREADS) {
+                *n_threads = OVN_MAX_SUPPORTED_THREADS;
+                VLOG_WARN("Setting n_threads to %d as --n-threads option was "
+                    "set to : [%s]", *n_threads, optarg);
+            }
+            if (*n_threads != 1) {
+                VLOG_INFO("Using %d threads", *n_threads);
+            }
             break;
 
         case OPT_DRY_RUN:
@@ -673,13 +697,44 @@ get_probe_interval(const char *db, const struct nbrec_nb_global *nb)
     return interval;
 }
 
+static struct ovsdb_idl_txn *
+run_idl_loop(struct ovsdb_idl_loop *idl_loop, const char *name)
+{
+    unsigned long long duration, start = time_msec();
+    unsigned int seqno = UINT_MAX;
+    struct ovsdb_idl_txn *txn;
+    int n = 0;
+
+    /* Accumulate database changes as long as there are some,
+     * but no longer than half a second. */
+    while (seqno != ovsdb_idl_get_seqno(idl_loop->idl)
+           && time_msec() - start < 500) {
+        seqno = ovsdb_idl_get_seqno(idl_loop->idl);
+        ovsdb_idl_run(idl_loop->idl);
+        n++;
+    }
+
+    txn = ovsdb_idl_loop_run(idl_loop);
+
+    duration = time_msec() - start;
+    /* ovsdb_idl_run() is called at least 2 times.  Once directly and
+     * once in the ovsdb_idl_loop_run().  n > 2 means that we received
+     * data on at least 2 subsequent calls. */
+    if (n > 2 || duration > 100) {
+        VLOG(duration > 500 ? VLL_INFO : VLL_DBG,
+             "%s IDL run: %d iterations in %lld ms", name, n + 1, duration);
+    }
+    return txn;
+}
+
 int
 main(int argc, char *argv[])
 {
     int res = EXIT_SUCCESS;
     struct unixctl_server *unixctl;
     int retval;
-    struct ovn_exit_args exit_args = {};
+    bool exiting;
+    int n_threads = 1;
     struct northd_state state = {
         .had_lock = false,
         .paused = false
@@ -689,7 +744,7 @@ main(int argc, char *argv[])
     ovs_cmdl_proctitle_init(argc, argv);
     ovn_set_program_name(argv[0]);
     service_start(&argc, &argv);
-    parse_options(argc, argv, &state.paused);
+    parse_options(argc, argv, &state.paused, &n_threads);
 
     daemonize_start(false);
 
@@ -700,8 +755,7 @@ main(int argc, char *argv[])
     if (retval) {
         exit(EXIT_FAILURE);
     }
-    unixctl_command_register("exit", "", 0, 0, ovn_exit_command_callback,
-                             &exit_args);
+    unixctl_command_register("exit", "", 0, 0, ovn_northd_exit, &exiting);
     unixctl_command_register("pause", "", 0, 0, ovn_northd_pause, &state);
     unixctl_command_register("resume", "", 0, 0, ovn_northd_resume, &state);
     unixctl_command_register("is-paused", "", 0, 0, ovn_northd_is_paused,
@@ -717,6 +771,12 @@ main(int argc, char *argv[])
     unixctl_command_register("nb-cluster-state-reset", "", 0, 0,
                              cluster_state_reset_cmd,
                              &reset_ovnnb_idl_min_index);
+    unixctl_command_register("parallel-build/set-n-threads", "N_THREADS", 1, 1,
+                             ovn_northd_set_thread_count_cmd,
+                             NULL);
+    unixctl_command_register("parallel-build/get-n-threads", "", 0, 0,
+                             ovn_northd_get_thread_count_cmd,
+                             NULL);
 
     daemonize_complete();
 
@@ -774,10 +834,13 @@ main(int argc, char *argv[])
     unsigned int ovnnb_cond_seqno = UINT_MAX;
     unsigned int ovnsb_cond_seqno = UINT_MAX;
 
+    run_update_worker_pool(n_threads);
+
     /* Main loop. */
+    exiting = false;
 
     bool recompute = false;
-    while (!exit_args.exiting) {
+    while (!exiting) {
         update_ssl_config();
         memory_run();
         if (memory_should_report()) {
@@ -800,8 +863,8 @@ main(int argc, char *argv[])
                 ovsdb_idl_set_lock(ovnsb_idl_loop.idl, "ovn_northd");
             }
 
-            struct ovsdb_idl_txn *ovnnb_txn =
-                        ovsdb_idl_loop_run(&ovnnb_idl_loop);
+            struct ovsdb_idl_txn *ovnnb_txn = run_idl_loop(&ovnnb_idl_loop,
+                                                           "OVN_Northbound");
             unsigned int new_ovnnb_cond_seqno =
                         ovsdb_idl_get_condition_seqno(ovnnb_idl_loop.idl);
             if (new_ovnnb_cond_seqno != ovnnb_cond_seqno) {
@@ -812,8 +875,8 @@ main(int argc, char *argv[])
                 ovnnb_cond_seqno = new_ovnnb_cond_seqno;
             }
 
-            struct ovsdb_idl_txn *ovnsb_txn =
-                        ovsdb_idl_loop_run(&ovnsb_idl_loop);
+            struct ovsdb_idl_txn *ovnsb_txn = run_idl_loop(&ovnsb_idl_loop,
+                                                           "OVN_Southbound");
             unsigned int new_ovnsb_cond_seqno =
                         ovsdb_idl_get_condition_seqno(ovnsb_idl_loop.idl);
             if (new_ovnsb_cond_seqno != ovnsb_cond_seqno) {
@@ -909,7 +972,7 @@ main(int argc, char *argv[])
         unixctl_server_run(unixctl);
         unixctl_server_wait(unixctl);
         memory_wait();
-        if (exit_args.exiting) {
+        if (exiting) {
             poll_immediate_wake();
         }
 
@@ -940,21 +1003,30 @@ main(int argc, char *argv[])
         stopwatch_stop(NORTHD_LOOP_STOPWATCH_NAME, time_msec());
         poll_block();
         if (should_service_stop()) {
-            exit_args.exiting = true;
+            exiting = true;
         }
         stopwatch_start(NORTHD_LOOP_STOPWATCH_NAME, time_msec());
     }
     inc_proc_northd_cleanup();
 
+    unixctl_server_destroy(unixctl);
     ovsdb_idl_loop_destroy(&ovnnb_idl_loop);
     ovsdb_idl_loop_destroy(&ovnsb_idl_loop);
-    ovn_exit_args_finish(&exit_args);
-    unixctl_server_destroy(unixctl);
     service_stop();
 
     exit(res);
 }
 
+static void
+ovn_northd_exit(struct unixctl_conn *conn, int argc OVS_UNUSED,
+                const char *argv[] OVS_UNUSED, void *exiting_)
+{
+    bool *exiting = exiting_;
+    *exiting = true;
+
+    unixctl_command_reply(conn, NULL);
+}
+
 static void
 ovn_northd_pause(struct unixctl_conn *conn, int argc OVS_UNUSED,
                 const char *argv[] OVS_UNUSED, void *state_)
@@ -1020,3 +1092,30 @@ cluster_state_reset_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
     poll_immediate_wake();
     unixctl_command_reply(conn, NULL);
 }
+
+static void
+ovn_northd_set_thread_count_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
+               const char *argv[], void *aux OVS_UNUSED)
+{
+    int n_threads = atoi(argv[1]);
+
+    if ((n_threads < 1) || (n_threads > OVN_MAX_SUPPORTED_THREADS)) {
+        struct ds s = DS_EMPTY_INITIALIZER;
+        ds_put_format(&s, "invalid n_threads: %d\n", n_threads);
+        unixctl_command_reply_error(conn, ds_cstr(&s));
+        ds_destroy(&s);
+    } else {
+        run_update_worker_pool(n_threads);
+        unixctl_command_reply(conn, NULL);
+    }
+}
+
+static void
+ovn_northd_get_thread_count_cmd(struct unixctl_conn *conn, int argc OVS_UNUSED,
+               const char *argv[] OVS_UNUSED, void *aux OVS_UNUSED)
+{
+    struct ds s = DS_EMPTY_INITIALIZER;
+    ds_put_format(&s, "%"PRIuSIZE"\n", get_worker_pool_size());
+    unixctl_command_reply(conn, ds_cstr(&s));
+    ds_destroy(&s);
+}
